package org.ow2.bonita.example.aw;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;

import org.ow2.bonita.example.AbstractExample;
import org.ow2.bonita.facade.QueryDefinitionAPI;
import org.ow2.bonita.facade.QueryRuntimeAPI;
import org.ow2.bonita.facade.RuntimeAPI;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.runtime.ActivityInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.runtime.TaskState;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.bonita.util.AccessorUtil;
import org.ow2.bonita.util.BonitaException;
import org.ow2.bonita.util.BonitaRuntimeException;
import org.ow2.novabpm.util.Misc;
import org.ow2.novabpm.util.SimpleCallbackHandler;
import org.ow2.novabpm.util.StandardCallbackHandler;

/**
 * 
 * @author Thomas Gueze
 *
 */
public class MultiInstantiationApproval extends AbstractExample {

  private static final Logger LOG = Logger.getLogger(MultiInstantiationApproval.class.getName());
  private static Map<String, String> users = new HashMap<String, String>();
  private static Map<String, String> votes = new HashMap<String, String>();

  @Override
  public void afterInstantiate(ProcessInstanceUUID instanceUUID,
    ProcessDefinitionUUID processUUID, RuntimeAPI runtimeAPI,
    QueryRuntimeAPI queryRuntimeAPI, QueryDefinitionAPI queryDefinitionAPI)
    throws BonitaException {
    throw new AssertionError("This method should not be called");
  }
  
  public void performTask(ProcessInstanceUUID instanceUUID, RuntimeAPI runtimeAPI,
    QueryRuntimeAPI queryRuntimeAPI, String vote)
    throws BonitaException {
    final Collection<ActivityInstance< TaskInstance >> activities = queryRuntimeAPI.getTaskList(instanceUUID, TaskState.READY);

    if (activities.isEmpty()) {
      throw new BonitaRuntimeException("No task found? Bad User?");
    }
    for (ActivityInstance< TaskInstance > activity : activities) {
      final TaskUUID taskUUID = activity.getBody().getUUID();
      final String activityId = activity.getActivityId();
      if (LOG.isLoggable(Level.INFO)) {
        LOG.fine(Misc.LINE_SEPARATOR 
            + "***** Starting task associated to activity: " + activityId + " *****" 
            + Misc.LINE_SEPARATOR);
      }
      runtimeAPI.startTask(taskUUID, true);
      runtimeAPI.setActivityInstanceVariable(activity.getUUID(),
          "decisionAccepted", vote);
      if (LOG.isLoggable(Level.INFO)) {
        LOG.fine(Misc.LINE_SEPARATOR + "***** Finishing task associated to activity: " + activityId + " *****"
            + Misc.LINE_SEPARATOR);
      }
      runtimeAPI.finishTask(taskUUID, true);
      if (LOG.isLoggable(Level.INFO)) {
        LOG.fine(Misc.LINE_SEPARATOR 
            + "***** Task associated to activity: " + activityId + " finished. *****"
            + Misc.LINE_SEPARATOR);
      }
    }
  }

  @Override
  public String getProcessId() {
    return "multiInstantiation";
  }
  
  public void login(String userName, String password) throws LoginException {
    String loginMode = System.getProperty("login.mode");
    if (loginMode != null && "test".equals(loginMode)) {
      loginContext = new LoginContext("Bonita", new SimpleCallbackHandler(userName, password));
    } else {
      loginContext = new LoginContext("Bonita", new StandardCallbackHandler());
    }
    loginContext.login();
  }
  
  public static ProcessInstanceUUID runApproval(String barFile, int numberOfAccepted) throws Exception {
    users.put("John", "bonita");
    users.put("Jack", "bonita");
    users.put("James", "jamespwd");
    switch (numberOfAccepted) {
      case 0:
        votes.put("John", "no");
        votes.put("Jack", "no");
        votes.put("James", "no");
        break;
      case 1 :
        votes.put("John", "yes");
        votes.put("Jack", "no");
        votes.put("James", "no");
        break;
      case 2 :
        votes.put("John", "yes");
        votes.put("Jack", "yes");
        votes.put("James", "no");
        break;
      default:
      case 3 :
        votes.put("John", "yes");
        votes.put("Jack", "yes");
        votes.put("James", "yes");
        break;
    }
    
    final MultiInstantiationApproval multiInstantiation = new MultiInstantiationApproval();
    multiInstantiation.login();
    ProcessDefinition process = multiInstantiation.deploy(AccessorUtil.getManagementAPI(), barFile)
      .get(multiInstantiation.getProcessId());
    ProcessInstanceUUID instanceUUID = AccessorUtil.getRuntimeAPI().instantiateProcess(process.getUUID());
    multiInstantiation.logout();
    String user = null;
    for (Iterator<String> userIterator = users.keySet().iterator();userIterator.hasNext();user = userIterator.next()) {
      multiInstantiation.login(user, users.get(user));
      multiInstantiation.performTask(instanceUUID,
        AccessorUtil.getRuntimeAPI(), AccessorUtil.getQueryRuntimeAPI(), votes.get(user));
      multiInstantiation.logout();
    }
    return instanceUUID;
  }
  
  public static void main(final String[] args) throws Exception {
    if (args == null || args.length != 2) {
      System.err.println("Usage: java " + MultiInstantiationApproval.class.getName() + " barFile numberOfAccept");
      throw new BonitaRuntimeException("Please read the above usage");
    }
    final String barFile = args[0];
    final int numberOfAccept = Integer.parseInt(args[1]);
    runApproval(barFile, numberOfAccept);
  }
}
